//Contains all of the routines needed to load settings from the zephyr settings backend. 

//Defualts for settings
//These will be applied if a valid key/value pair is not found when the settings module is loaded up

#ifndef SETTINGS_MODULE_H
#define SETTINGS_MODULE_H

#include <zephyr/settings/settings.h>
#include <stdbool.h>
#include "boardDefine.h"

#define DEF_ACTIVE_TRANSMIT_INTERVAL 86400 // 1 Day
#define DEF_NONACTIVE_TRANSMIT_INTERVAL 604800 // 1 Week

#define DEF_ACTIVE_MEASUREMENT_SPACING 3600 // 1 Hour
#define DEF_NONACTIVE_MEASUREMENT_SPACING 43200 // 12 Hours
#define DEF_SENSOR_ENABLED 0
#define DEF_DISABLE_DURING_NONACTIVE 0

#define DEF_SYSTEM_NAME "Unamed_System"
#define DEF_LTE_ENABLED true

//Settings Structs
typedef struct powerStateSettings{
    int activeSeasonStartMonth;
    int nonActiveSeasonStartMonth;
} powerStateSettings;

typedef struct communicationSettings{
    int activeTransmitInterval;
    int nonActiveTransmitInterval;
} communicationSettings;

typedef struct systemSettings{
    char systemName[100];
    bool lteEnabled;
} systemSettings;

//Struct isntances
extern powerStateSettings powerSettings;
extern communicationSettings comSettings;
extern systemSettings sysSettings;

//Handler fucntions
//System
int system_handle_set(const char *name, size_t len, settings_read_cb read_cb, void *cb_arg);
int system_handle_export(int (*cb)(const char *name, const void *value, size_t val_len));
//Power States
int powerState_handle_set(const char *name, size_t len, settings_read_cb read_cb, void *cb_arg);
int powerState_handle_export(int (*cb)(const char *name, const void *value, size_t val_len));


//External usage functions
int init_settings();

#endif //End of header guard #ifndef