#include <zephyr/kernel.h>
#include <zephyr/devicetree.h>
#include <zephyr/drivers/flash.h>
#include <zephyr/pm/pm.h>
#include <zephyr/drivers/gpio.h>
#include <zephyr/posix/unistd.h>
//Custom Modules
#include "includes/RTCHelper.h"
#include "includes/connectionManager.h"
#include "includes/MeasurementSystem.h"
#include "includes/batteryCharger.h"
#include "includes/timerInteruptHandler.h"
#include "includes/boardDefine.h"

#define DEBUGGING

LOG_MODULE_REGISTER(firmware_log);

//Interupt init functions
//As it requires access to so many modules it makes sense to leave here defined below main
void initBoardInterupts();

int main(void)
{
        //Start system Initilzation
        Networkconnect();
        batteryChargerInit();
        //RTC
        RTCdisableCLKout();
        uint64_t timeBuffer;
        RTCgetTime(&timeBuffer);
        if (boardRTCState->timeValid == false){ //Power loss has occured and RTC needs to be reloaded via the network, start the connection process
                while(true){ //Loop until the network is ready (Maybe add more handling here later)
                        if (NetworkState->networkActive == true){
                                NetworksyncRTCClockSNTP();
                                break;
                        }
                        else {k_msleep(10000);} //Check every 10s if the network is up and ready
                }
        }
        //Continued
        initBoardInterupts();
        MeasurementInit();
        interuptHandlerInit();
        //Network Services
        while(true){ //Need the network to init cloud services, check every 500ms
                if (NetworkState->networkActive == true){
                        break;
                }
                k_msleep(500);
        }
        Network_initCloud();
        //In deployment we need to jump right into the interupts
        #ifndef DEBUGGING
        
        #endif
        return 0;
}

//Init of all board interupts
void initBoardInterupts(){
        LOG_MODULE_DECLARE(firmware_log, LOG_LEVEL_INF);
        LOG_INF("Assigning pin interupts");
        //Configure task system interupt pin
        gpio_pin_configure_dt(&RTC_Alarm_GPIO_DT, GPIO_INPUT);
        gpio_pin_interrupt_configure_dt(&RTC_Alarm_GPIO_DT, GPIO_INT_EDGE_FALLING);
        gpio_init_callback(&RTC_Alarm_CB, timer_alarm_isr, BIT(RTC_Alarm_GPIO_DT.pin));
        gpio_add_callback_dt(&RTC_Alarm_GPIO_DT, &RTC_Alarm_CB);
        LOG_INF("Finished assigning pin interupts");
}